/* 
   borZoi - An Elliptic Curve Cryptography Library

   Copyright (C) 2001 Anthony Mulcahy

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

*/


#include "borzoi.h"    // Include this to use the elliptic curve and
                       // symmetric cipher routines
#include "nist_curves.h" // Include this to use the curves recommended by NIST


// Elliptic Curve Key Agreement Scheme Diffie-Hellman
void eckas_dh1_ex () {
	// Degree 163 Binary Field from fips186-2
	use_NIST_B_163 ();
	EC_Domain_Parameters dp = NIST_B_163;

	ECSecKey sk1 (dp); // machine 1
	ECPubKey pk1 (sk1); // machine 1

	ECSecKey sk2 (dp); // machine 2
	ECPubKey pk2 (sk2); // machine 2

	// DER encoding, is a common standard method of representing public 
	// keys
	DER der1 (pk1); // machine 1
	DER der2 (pk2); // machine 2

	// You might save the DER ecoded keys to files or send them over 
	// the network here.
	std::cout << "DER Encoding of pk1.W: " << der1 << std::endl; // send to machine 2
	Point W2 = (der2.toECPubKey()).W;
	std::cout << "der2: " << W2 << std::endl;
	std::cout << "DER Encoding of pk2.W: " << der2 << std::endl; // send to machine 1
	try { // try to catch any DER parsing errors
		// User one generates KA from his secret key and user two's 
		// public key
		BigInt KA1 = OS2IP (ECKAS_DH1 (dp, sk1.s, pk2.W));
		BigInt KA = OS2IP (ECKAS_DH1 (dp, sk1.s, (der2.toECPubKey()).W));

		// User two generates KB from his secret key and user one's 
		// public key
		BigInt KB1 = OS2IP (ECKAS_DH1 (dp, sk2.s, pk1.W));
		BigInt KB = OS2IP (ECKAS_DH1 (dp, sk2.s, (der1.toECPubKey()).W));

		// Verify that the keys match 
		// (typically, users 1 and 2 would use KA and KB to encrypt
		//  data with a symmetric cipher for transimission to each 
		//  other here)
		std::cout << "sk1.s:" << sk1.s << std::endl;
		std::cout << "sk2.s:" << sk2.s << std::endl;
		std::cout << "pk1.W:" << pk1.W << std::endl;
		std::cout << "pk2.W:" << pk2.W << std::endl;
		//std::cout << "der1: " << (der1.toECPubKey()).W << std::endl;
		//std::cout << "der2: " << (der2.toECPubKey()).W << std::endl;
		std::cout << "KA1: " << KA1 << std::endl;
		std::cout << "KB1: " << KB1 << std::endl;
		std::cout << "KA:   " << KA << std::endl;
		std::cout << "KB:   " << KB << std::endl;
		if (KA == KB)
			std::cout << "Keys match\n";
		else std::cout << "Error keys don't match\n";
	} catch (borzoiException e) { // print the error message and exit
		e.debug_print ();
	}
}


// Elliptic Curve Digital Signature Algorithm Example
void ecdsa_ex () {
	// Degree 163 Binary Field from fips186-2
	use_NIST_B_163 ();
	EC_Domain_Parameters dp = NIST_B_163;

	std::cout << "r: " << dp.r << " (" << dp.r.numBits() << ")" << std::endl;
	ECSecKey sk (dp);// generate a key pair from the EC domain parameters

	std::cout << "sk.s: " << sk.s << " (" << sk.s.numBits() << ")" << std::endl;
	std::string M ("correct message");

	ECDSA sig1 (sk, OS2IP(SHA1(M))); // generate the signature

	// DER encoding, is a common standard method of representing digital 
	// signatures
	DER der_str (sig1);

	// You might save the DER ecoded signature to a file or send it over 
	// the network here.
	std::cout << "DER Encoding: " << der_str << std::endl;
	
	ECDSA sig2;
	try { // try to catch any DER parsing errors
		sig2 = der_str.toECDSA (); // decode the DER string
	} catch (borzoiException e) { // print the error message and exit
		e.debug_print ();
		return;
	}

	ECPubKey pk (sk);
	std::cout << "Checking signature against M: " << M.c_str () << "\n->";

	std::cout << "SHA1(M): " << OS2IP(SHA1(M)) << std::endl;
	if (sig2.verify(pk, OS2IP(SHA1(M)))) // try to verify the signature
		std::cout << "valid signature\n";
	else std::cout << "invalid signature\n";

	M = "in" + M; // tamper with the message
	std::cout << "Checking signature against M: " << M.c_str () << "\n->";

	if (sig2.verify(pk, OS2IP(SHA1(M)))) // try to verify the signature
		std::cout << "valid signature\n";
	else std::cout << "invalid signature\n";
}


// Elliptic Curve Integrated Encryption Scheme
void ecies_ex () {
	// Degree 163 Binary Field from fips186-2
	use_NIST_B_233 ();
	EC_Domain_Parameters dp = NIST_B_233;

	ECSecKey sk (dp);// generate a secret key from the EC domain parameters
	ECPubKey pk (sk);// calculate the public key the secret key

	// plaintext p1 = a, b, c, d
	OCTETSTR p1(4); p1[0]='a'; p1[1]='b'; p1[2]='c'; p1[3]='d';
	int i;
	for (i=0; i<4; i++) {
		std::cout << p1[i];
	}
	std::cout << std::endl;

	ECIES ct1 (p1, pk); // encrypt using the public key

	std::cout << "ct1: " << std::endl << ct1 << std::endl;
	OCTETSTR p2;
	try { // try to catch any exceptions if the tag is invalid
		p2 = ct1.decrypt(sk); // decrypt using the secret key
	} catch (borzoiException e) { // print the error message and exit
		e.debug_print ();
		return;
	}
	for (i=0; i<4; i++) {
		std::cout << p2[i];
	}
	std::cout << std::endl;


	// DER encoding
	DER der_str (ct1);

	// You might save the DER ecoded signature to a file or send it over 
	// the network here.
	std::cout << "DER Encoding: " << der_str << std::endl;
	
	ECIES ct2;
	try { // try to catch any DER parsing errors
		ct2 = der_str.toECIES (); // decode the DER string
	} catch (borzoiException e) { // print the error message and exit
		e.debug_print ();
		return;
	}

	std::cout << "ct2: " << std::endl << ct2 << std::endl;

	OCTETSTR p3;
	try { // try to catch any exceptions if the tag is invalid
		p3 = ct2.decrypt(sk); // decrypt using the secret key
	} catch (borzoiException e) { // print the error message and exit
		e.debug_print ();
		return;
	}
	for (i=0; i<4; i++) {
		std::cout << p3[i];
	}
	std::cout << std::endl;

}


// Symmetric Cipher Example
void symcipher_ex () {
	int keysize = 128; // 128 bit symmetric encryption key

	int i;
	OCTETSTR P1; // This are 0 for the time being (FIXME?)
	OCTETSTR Z(keysize/8);
	for (i=0; i<(keysize/8); i++) {
		Z[i] = (OCTET)gen_random ();
	}
	OCTETSTR key = KDF2 (Z, keysize/8, P1);

	// plaintext p1 = a, b, c, d
	OCTETSTR p1(4); p1[0]='a'; p1[1]='b'; p1[2]='c'; p1[3]='d';
	for (i=0; i<4; i++) {
		std::cout << p1[i];
	}
	std::cout << std::endl;

	// encrypt p1 to produce ciphertext c
	OCTETSTR c = AES_CBC_IV0_Encrypt (key, p1, keysize);
	for (i=0; i<4; i++) {
		std::cout << c[i];
	}
	std::cout << std::endl;

	// decrypt to produce plaintext p2
	OCTETSTR p2;
	try { // try to catch any exceptions
		p2 = AES_CBC_IV0_Decrypt (key, c, keysize);
	} catch (borzoiException e) { // print the error message and exit
		e.debug_print ();
		return;
	}
	for (i=0; i<4; i++) {
		std::cout << p2[i];
	}
	std::cout << std::endl;
}


int main (int argc, char* argv[]) {

	// Examples of using the ECKAS, ECDSA ECIES and symCipher classes

	eckas_dh1_ex (); // Elliptic Curve Key Agreement Scheme Diffie-Hellman
	ecdsa_ex (); // Elliptic Curve Digital Signature Algorithm
	ecies_ex (); // Elliptic Curve Integrated Encryption Scheme

	//symcipher_ex (); // Symmetric Cipher

	return 0;
}

